package jalview.gui;

import java.awt.*;
import javax.swing.*;
import java.util.*;
import jalview.datamodel.*;
import jalview.schemes.*;
import jalview.analysis.*;


public class SeqCanvas extends JPanel
{
    Image             img;
    Graphics          gg;
    int               imgWidth;
    int               imgHeight;

    AlignViewport     av;
    int pady = 2;
    int oldstartx;
    int oldstarty;
    int oldendx;
    int oldendy;

    public boolean paintFlag = false;
    boolean showScores = false;

    int chunkHeight;
    int chunkWidth;

    Graphics debugG;
    FeatureRenderer fr = new FeatureRenderer();
    BlockRenderer   br = new BlockRenderer();
    CpGRenderer     cgr = new CpGRenderer();

    ColourSchemeI    cs = new ZappoColourScheme();

    public SeqCanvas(AlignViewport av)
    {
	this.av         = av;
       setLayout(new BorderLayout());
    }

  public void drawScale(int startx, int endx,double charWidth, int charHeight,int ypos) {
      int scalestartx = startx - startx%10 + 10;

      gg.setColor(Color.black);

      for (int i=scalestartx;i < endx;i+= 10) {
	  String string = String.valueOf(i);
	  gg.drawString(string,(int)((i-startx-1)*charWidth),ypos+15 - charHeight*(2));
      }
  }


/**
 * Definitions of startx and endx (hopefully):
 * SMJS This is what I'm working towards!
 *   startx is the first residue (starting at 0) to display.
 *   endx   is the last residue to display (starting at 0).
 *   starty is the first sequence to display (starting at 0).
 *   endy   is the last sequence to display (starting at 0).
 * NOTE 1: The av limits are set in setFont in this class and
 * in the adjustment listener in SeqPanel when the scrollbars move.
 */

  public void paintComponent(Graphics g) {
    debugG = g;

    AlignmentI da = av.getAlignment();

    if (img == null ||
        imgWidth  !=  getWidth()  ||
        imgHeight !=  getHeight() ||
        paintFlag == true) {

      imgWidth  = (getWidth() > 0 ? getWidth() : 1);
      imgHeight = (getHeight() > 0 ? getHeight() : 1);

      img = createImage(imgWidth,imgHeight);
      gg  = img.getGraphics();

     // SMJS I added this in to update the AV when the size changes
      //      until I figure out how this should be done
      setFont(av.getFont());

      gg.setFont(av.getFont());

      paintFlag = false;

      oldstartx = -1;
      oldendx   = -1;
      oldstarty = -1;
      oldendy   = -1;
    }

    int startx = av.getStartRes();
    int starty = av.getStartSeq();

    int endx   = av.getEndRes();
    int endy   = av.getEndSeq();

    double charWidth  = av.getCharWidth();
    int charHeight    = av.getCharHeight();

    chunkWidth = (int)(getWidth()/charWidth);
    chunkHeight =  (da.getHeight() + 2)*charHeight;

    av.setChunkHeight(chunkHeight);
    av.setChunkWidth(chunkWidth);

    int offy = av.getStartSeq();

    if (oldendx == -1) {
      fillBackground(gg,Color.WHITE,0,0,imgWidth,imgHeight);

     /* if (av.getWrapAlignment() == true) {
	  startx = (int)(offy/chunkWidth)*chunkWidth;
	  endx   = startx + chunkWidth;
	  starty = offy%chunkHeight;
	  endy   = starty + da.getHeight();

	  int ypos     = 0;
	  int rowstart = starty;

	  if (starty == 0) {
	      ypos = 2*charHeight;
	  } else if (starty == 1) {
	      starty = 0;
	      ypos = charHeight;
	  }

	  if (endy > da.getHeight()) {
	      endy = da.getHeight();
	  }

	  if (endx > da.getWidth()) {
	      endx = da.getWidth();
	  }

	  if (rowstart < 2) {
	      drawScale(startx,endx,charWidth,charHeight,ypos);
	  }

	  drawPanel(gg,startx,endx,starty,endy,startx,starty,ypos);

	  if (rowstart == 0) {
	      ypos = ypos + chunkHeight;
	  } else if (rowstart == 1) {
	      ypos = ypos + chunkHeight;
	  } else {
	      ypos   = ypos + chunkHeight - rowstart*charHeight;
	  }

	  startx += chunkWidth;
	  endx   = startx + chunkWidth;
	  starty = 0;

	  if (endx > da.getWidth()) {
	      endx = da.getWidth();
	  }
	  // Draw the rest of the panels

	  while (ypos <= getHeight()) {
	      drawScale(startx,endx,charWidth,charHeight,ypos);
	      drawPanel(gg,startx,endx,0,da.getHeight(),startx,starty,ypos);

	      ypos   += chunkHeight;
	      startx += chunkWidth;
	      endx   = startx + chunkWidth;

	      if (endy > da.getHeight()) {
		  endy = da.getHeight();
	      }

	      if (endx > da.getWidth()) {
		  endx = da.getWidth();
	      }

	  }
      } else*/ {
	  drawPanel(gg,startx,endx,starty,endy,startx,starty,0);

	  oldstartx = startx;
	  oldendx   = endx;
	  oldstarty = starty;
	  oldendy   = endy;

      }
    }

    else if (oldstartx < startx)
    {
      // This is dragging horizontal scrollbar to the right

      int delx  = (int)((startx - oldstartx) * charWidth);
      int delx2 = (int)((oldendx - startx + 1)   * charWidth);

      gg.copyArea(delx,0,delx2,AlignmentUtil.getPixelHeight(starty,endy,charHeight),-delx,0);

      if (startx > oldendx)
	drawPanel(gg,startx,endx,starty,endy,startx,starty,0);
      else
	drawPanel(gg,oldendx+1,endx,starty,endy,startx,starty,0);


      oldstartx = startx;
      oldendx   = endx;

    } else if (oldstartx > startx)
    {
     // Horizontal scrollbar pulled to the left

      int delx  = (int)((oldstartx - startx) * charWidth);
      int delx2 = (int)((endx - oldstartx +1)   * charWidth);

      gg.copyArea(0,0,delx2,AlignmentUtil.getPixelHeight(starty,endy,charHeight),delx,0);

      if (oldstartx > endx) {
	drawPanel(gg,startx,endx,starty,endy,startx,starty,0);
      } else {
	drawPanel(gg,startx,oldstartx-1,starty,endy,startx,starty,0);
      }

      oldstartx = startx;
      oldendx   = endx;

    }  else if (oldstarty < starty) {
      // Vertical scrollbar down
      int dely  = AlignmentUtil.getPixelHeight(oldstarty,starty,charHeight);
      int dely2 = AlignmentUtil.getPixelHeight(starty,oldendy,charHeight);

      gg.copyArea(0,dely,(int)((endx-startx+1)*charWidth),dely2,0,-dely);

      if (starty > oldendy) {
	drawPanel(gg,startx,endx,starty,endy,startx,starty,0);
      } else {
	drawPanel(gg,startx,endx,oldendy,endy,startx,starty,0);
      }

      oldstarty = starty;
      oldendy   = endy;

    } else if (oldstarty > starty) {

      // Vertical scrollbar up
      int dely  = AlignmentUtil.getPixelHeight(endy,oldendy,charHeight);
      int dely2 = AlignmentUtil.getPixelHeight(oldstarty,endy,charHeight);

      gg.copyArea(0,0,(int)((endx-startx+1)*charWidth),dely2,0,dely);

      if (oldstarty > endy) {
	drawPanel(gg,startx,endx,starty,endy,startx,starty,0);
      } else {
	drawPanel(gg,startx,endx,starty,oldstarty,startx,starty,0);
      }

      oldstarty = starty;
      oldendy   = endy;
    }

    if ((oldendy -oldstarty) > (int)((getWidth() / av.getCharWidth()))) {
      System.out.println("LIMITS ERROR LIMITS ERROR");
      System.out.println("Corrds " + (oldendy-oldstarty) + " " + (int)(getWidth()/av.getCharWidth()) + " " + getWidth() + " " + av.getCharWidth());
    }


    g.drawImage(img,0,0,this);

  }




  public void drawPanel(Graphics g,int x1,int x2, int y1, int y2,int startx, int starty,int offset) {

/*
    System.out.println("drawPanel called with g      = " + g);
    System.out.println("                      x1     = " + x1);
    System.out.println("                      x2     = " + x2);
    System.out.println("                      y1     = " + y1);
    System.out.println("                      y2     = " + y2);
    System.out.println("                      startx = " + startx);
    System.out.println("                      starty = " + starty);
*/

    g.setFont(av.getFont());
    double            charWidth  = av.getCharWidth();
    int               charHeight = av.getCharHeight();
    RendererI sr = av.getRenderer();


    /*Vector    pid    = av.getConsensus(false);

    Vector tmpseq = new Vector();

    for (int i = 0; i < av.getAlignment().getHeight(); i++)
	if (!av.getSelection().contains(av.getAlignment().getSequenceAt(i)))
	    tmpseq.addElement(av.getAlignment().getSequenceAt(i));



    if (sr instanceof SequenceRenderer)
	pid    = AAFrequency.calculate(tmpseq,x1,x2);

    else if (sr instanceof GraphRenderer)
	pid = AAFrequency.calculatePID(av.getAlignment().getSequenceAt(0),
				       av.getAlignment().getSequences(),
				       av.getPIDWindow(),x1,x2);

*/

    if (y2 > starty && y1 < av.getEndSeq()) {

    fillBackground(g,
                   Color.white,
                   (int)((x1-startx)*charWidth),
                   offset + AlignmentUtil.getPixelHeight(starty,y1,av.getCharHeight()),
                   (int)((x2-x1+1)*charWidth),
                   offset + AlignmentUtil.getPixelHeight(y1,y2,av.getCharHeight()));
    }

    for (int i = y1 ; i < y2 ;i++) {

      RendererI r = sr;
/*
      if (av.getSelection().contains(av.getAlignment().getSequenceAt(i))) {
          r = fr;
          System.out.println("use feature renderer");
      } else if ( i == 0) {
        //  r = br;
      } else if (av.getAlignment().getSequenceAt(i).getName().equals("CpG")) {
          r = cgr;
          System.out.println("cg renderer");
      }
*/

      r.drawSequence(g,cs,av.getAlignment().getSequenceAt(i),
                      x1,
                      x2,
                      (int)((x1-startx)*charWidth),
                      offset + AlignmentUtil.getPixelHeight(starty,i,av.getCharHeight()),
                      charWidth,
                      charHeight,showScores,av.getShowBoxes(),av.getShowText(),null,i,av);

    }
  }



  public void fillBackground(Graphics g,Color c, int x1,int y1,int width,int height) {
    g.setColor(c);
    g.fillRect(x1,y1,width,height);
  }

  public int getChunkWidth() {
    return chunkWidth;
  }

}
